@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
 import json
@@ -8,12 +7,11 @@ from contextlib import contextmanager, suppress
 from functools import partial
 from typing import Any, Dict, Generator, Iterable, List, Optional, Tuple
 
-from .conf.utils import BadLine, load_config as _load_config, parse_config_base
+from .conf.utils import BadLine, parse_config_base
+from .conf.utils import load_config as _load_config
 from .constants import cache_dir, defconf
 from .options.types import Options, defaults, option_names
-from .options.utils import (
-    KeyDefinition, KeyMap, MouseMap, MouseMapping, SequenceMap
-)
+from .options.utils import KeyDefinition, KeyMap, MouseMap, MouseMapping, SequenceMap, build_action_aliases
 from .typing import TypedDict
 from .utils import log_error
 
@@ -22,9 +20,6 @@ def option_names_for_completion() -> Tuple[str, ...]:
     return option_names
 
 
-no_op_actions = frozenset({'noop', 'no-op', 'no_op'})
-
-
 def build_ansi_color_table(opts: Optional[Options] = None) -> int:
     if opts is None:
         opts = defaults
@@ -52,22 +47,20 @@ def atomic_save(data: bytes, path: str) -> None:
         except FileNotFoundError:
             pass
         except Exception as err:
-            log_error('Failed to delete temp file {} for atomic save with error: {}'.format(
-                p, err))
+            log_error(f'Failed to delete temp file {p} for atomic save with error: {err}')
 
 
 @contextmanager
-def cached_values_for(name: str) -> Generator[Dict, None, None]:
-    cached_path = os.path.join(cache_dir(), name + '.json')
-    cached_values: Dict = {}
+def cached_values_for(name: str) -> Generator[Dict[str, Any], None, None]:
+    cached_path = os.path.join(cache_dir(), f'{name}.json')
+    cached_values: Dict[str, Any] = {}
     try:
         with open(cached_path, 'rb') as f:
             cached_values.update(json.loads(f.read().decode('utf-8')))
     except FileNotFoundError:
         pass
     except Exception as err:
-        log_error('Failed to load cached in {} values with error: {}'.format(
-            name, err))
+        log_error(f'Failed to load cached in {name} values with error: {err}')
 
     yield cached_values
 
@@ -75,8 +68,7 @@ def cached_values_for(name: str) -> Generator[Dict, None, None]:
         data = json.dumps(cached_values).encode('utf-8')
         atomic_save(data, cached_path)
     except Exception as err:
-        log_error('Failed to save cached values with error: {}'.format(
-            err))
+        log_error(f'Failed to save cached values with error: {err}')
 
 
 def commented_out_default_config() -> str:
@@ -94,18 +86,25 @@ def prepare_config_file_for_editing() -> str:
     return defconf
 
 
-def finalize_keys(opts: Options) -> None:
+def finalize_keys(opts: Options, accumulate_bad_lines: Optional[List[BadLine]] = None) -> None:
     defns: List[KeyDefinition] = []
     for d in opts.map:
         if d is None:  # clear_all_shortcuts
             defns = []  # type: ignore
         else:
-            defns.append(d.resolve_and_copy(opts.kitty_mod, opts.kitten_alias))
+            try:
+                defns.append(d.resolve_and_copy(opts.kitty_mod))
+            except Exception as err:
+                if accumulate_bad_lines is None:
+                    log_error(f'Ignoring map with invalid action: {d.definition}. Error: {err}')
+                else:
+                    accumulate_bad_lines.append(BadLine(d.definition_location.number, d.definition_location.line, err, d.definition_location.file))
+
     keymap: KeyMap = {}
     sequence_map: SequenceMap = {}
 
     for defn in defns:
-        is_no_op = defn.action.func in no_op_actions
+        is_no_op = defn.is_no_op
         if defn.is_sequence:
             keymap.pop(defn.trigger, None)
             s = sequence_map.setdefault(defn.trigger, {})
@@ -114,32 +113,38 @@ def finalize_keys(opts: Options) -> None:
                 if not s:
                     del sequence_map[defn.trigger]
             else:
-                s[defn.rest] = defn.action
+                s[defn.rest] = defn.definition
         else:
             sequence_map.pop(defn.trigger, None)
             if is_no_op:
                 keymap.pop(defn.trigger, None)
             else:
-                keymap[defn.trigger] = defn.action
+                keymap[defn.trigger] = defn.definition
     opts.keymap = keymap
     opts.sequence_map = sequence_map
 
 
-def finalize_mouse_mappings(opts: Options) -> None:
+def finalize_mouse_mappings(opts: Options, accumulate_bad_lines: Optional[List[BadLine]] = None) -> None:
     defns: List[MouseMapping] = []
     for d in opts.mouse_map:
         if d is None:  # clear_all_mouse_actions
             defns = []  # type: ignore
         else:
-            defns.append(d.resolve_and_copy(opts.kitty_mod, opts.kitten_alias))
+            try:
+                defns.append(d.resolve_and_copy(opts.kitty_mod))
+            except Exception as err:
+                if accumulate_bad_lines is None:
+                    log_error(f'Ignoring mouse_map with invalid action: {d.definition}. Error: {err}')
+                else:
+                    accumulate_bad_lines.append(BadLine(d.definition_location.number, d.definition_location.line, err, d.definition_location.file))
     mousemap: MouseMap = {}
 
     for defn in defns:
-        is_no_op = defn.action.func in no_op_actions
+        is_no_op = defn.is_no_op
         if is_no_op:
             mousemap.pop(defn.trigger, None)
         else:
-            mousemap[defn.trigger] = defn.action
+            mousemap[defn.trigger] = defn.definition
     opts.mousemap = mousemap
 
 
@@ -162,13 +167,16 @@ def load_config(*paths: str, overrides: Optional[Iterable[str]] = None, accumula
     opts_dict, paths = _load_config(defaults, partial(parse_config, accumulate_bad_lines=accumulate_bad_lines), merge_result_dicts, *paths, overrides=overrides)
     opts = Options(opts_dict)
 
-    finalize_keys(opts)
-    finalize_mouse_mappings(opts)
+    opts.alias_map = build_action_aliases(opts.kitten_alias, 'kitten')
+    opts.alias_map.update(build_action_aliases(opts.action_alias))
+    finalize_keys(opts, accumulate_bad_lines)
+    finalize_mouse_mappings(opts, accumulate_bad_lines)
     # delete no longer needed definitions, replacing with empty placeholders
     opts.kitten_alias = {}
+    opts.action_alias = {}
     opts.mouse_map = []
     opts.map = []
-    if opts.background_opacity < 1.0 and opts.macos_titlebar_color:
+    if opts.background_opacity < 1.0 and opts.macos_titlebar_color > 0:
         log_error('Cannot use both macos_titlebar_color and background_opacity')
         opts.macos_titlebar_color = 0
     opts.config_paths = paths
